import {
  select,
  json,
  csv,
  tree,
  hierarchy,
  linkHorizontal,
  forceSimulation,
  forceLink,
  forceManyBody,
  forceCenter,
} from 'd3';

// Fetch the data from the URL
const dataURL =
  'https://raw.githubusercontent.com/Prasaadreddy/InternetSpeed/main/internet_speeds.csv';

// Preprocess the data for both tree and network graph
csv(dataURL)
  .then((data) => {
    // Preprocess data for the tree structure
    const treeData = processDataForTree(data);

    // Preprocess data for the network graph
    const networkData = processDataForNetwork(data);

    // Create visualizations using treeData and networkData
    createTreeVisualization(treeData);
    createNetworkGraphVisualization(networkData);
  })
  .catch((error) => {
    console.error('Error loading data:', error);
  });

// Function to preprocess data for tree structure
function processDataForTree(data) {
  const classifications = [
    ...new Set(data.map((d) => d.Classification)),
  ];
  const treeData = {
    name: 'Internet Speeds',
    children: classifications.map((c) => ({
      name: c,
      children: data
        .filter((d) => d.Classification === c)
        .map((d) => ({
          name: d.Country,
          value:
            d.Average_Broadband_Speed +
            d.Average_Mobile_Speed,
        })),
    })),
  };
  return treeData;
}

// Function to preprocess data for network graph
function processDataForNetwork(data) {
  const nodes = data.map((d) => ({
    id: d.Country,
    group: d.Classification,
  }));
  const links = [];
  data.forEach((d1) => {
    data.forEach((d2) => {
      if (d1.Country !== d2.Country) {
        links.push({
          source: d1.Country,
          target: d2.Country,
        });
      }
    });
  });
  return { nodes, links };
}

// Function to create a tree structure visualization
function createTreeVisualization(treeData) {
  const svg = select('#tree-svg');
  const width = +svg.attr('width');
  const height = +svg.attr('height');
  const margin = {
    top: 20,
    right: 160,
    bottom: 50,
    left: 140,
  };

  const treeLayout = tree().size([
    height - margin.top - margin.bottom,
    width - margin.left - margin.right,
  ]);

  const root = hierarchy(treeData);
  treeLayout(root);

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  g.selectAll('.link')
    .data(root.links())
    .enter()
    .append('path')
    .attr('class', 'link')
    .attr(
      'd',
      linkHorizontal()
        .x((d) => d.y)
        .y((d) => d.x),
    );

  const node = g
    .selectAll('.node')
    .data(root.descendants())
    .enter()
    .append('g')
    .attr('class', 'node')
    .attr('transform', (d) => `translate(${d.y},${d.x})`);

  node
    .append('circle')
    .attr('r', 4)
    .attr('fill', (d) =>
      d.data.name === 'Internet Speeds'
        ? 'black'
        : d.data.name === 'High'
          ? 'green'
          : d.data.name === 'Medium'
            ? 'orange'
            : 'red',
    );

  node
    .append('text')
    .attr('dy', '.35em')
    .attr('x', (d) => (d.children ? -13 : 13))
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name)
    .clone(true)
    .lower()
    .attr('stroke', 'white');
}

// Function to create a network graph visualization
function createNetworkGraphVisualization(networkData) {
  const svg = select('#network-svg');
  const width = +svg.attr('width');
  const height = +svg.attr('height');

  const simulation = forceSimulation(networkData.nodes)
    .force(
      'link',
      forceLink(networkData.links).id((d) => d.id),
    )
    .force('charge', forceManyBody().strength(-100))
    .force('center', forceCenter(width / 2, height / 2));

  const link = svg
    .selectAll('.link')
    .data(networkData.links)
    .enter()
    .append('line')
    .attr('class', 'link')
    .attr('stroke-width', (d) => Math.sqrt(d.value));

  const node = svg
    .selectAll('.node')
    .data(networkData.nodes)
    .enter()
    .append('circle')
    .attr('class', 'node')
    .attr('r', 5)
    .attr('fill', (d) =>
      d.group === 'High'
        ? 'green'
        : d.group === 'Medium'
          ? 'orange'
          : 'red',
    )
    .call(drag(simulation));

  node.append('title').text((d) => d.id);

  simulation.on('tick', () => {
    link
      .attr('x1', (d) => d.source.x)
      .attr('y1', (d) => d.source.y)
      .attr('x2', (d) => d.target.x)
      .attr('y2', (d) => d.target.y);

    node.attr('cx', (d) => d.x).attr('cy', (d) => d.y);
  });

  function drag(simulation) {
    function dragstarted(event, d) {
      if (!event.active)
        simulation.alphaTarget(0.3).restart();
      d.fx = d.x;
      d.fy = d.y;
    }

    function dragged(event, d) {
      d.fx = event.x;
      d.fy = event.y;
    }

    function dragended(event, d) {
      if (!event.active) simulation.alphaTarget(0);
      d.fx = null;
      d.fy = null;
    }

    return dragstarted, dragged, dragended;
  }
}
